# Copyright (C) 2013-2016 Freescale Semiconductor
# Copyright 2017-2021 NXP

DESCRIPTION = "i.MX U-Boot suppporting i.MX reference boards."
SECTION = "bootloaders"
DEPENDS += " \
    bc-native \
    bison-native \
    dtc-native \
    flex-native \
    gnutls-native \
        xxd-native \
"

require recipes-bsp/u-boot/u-boot.inc
inherit fsl-u-boot-localversion

S = "${WORKDIR}/git"
B = "${WORKDIR}/build"

do_configure[cleandirs] = "${B}"

FILESEXTRAPATHS:prepend := "${THISDIR}/files:"
PROVIDES += "u-boot"

LICENSE = "GPLv2+"
LIC_FILES_CHKSUM = "file://Licenses/gpl-2.0.txt;md5=b234ee4d69f5fce4486a80fdaf4a4263"

UBOOT_SRC ?= "git://gitlab.tastitalia.com/tastitalia-mpu-arm/yocto/bsp/u-boot-tastitalia-imx.git;protocol=https"

SRCBRANCH = "2022.04"
#SRCBRANCH:mx9-nxp-bsp = "2023.04"
SRC_URI = "${UBOOT_SRC};branch=${SRCBRANCH}"

SRCREV_default = "${AUTOREV}"
PV = "1.0+git${SRCPV}"

LOCALVERSION ?= "-${SRCBRANCH}"

BOOT_TOOLS = "imx-boot-tools"

SRC_URI:append:mx8mm7 = " file://tasd8mm-display-7.patch"
SRC_URI:append:mx8mm2g7 = " file://tasd8mm-2g-display-7.patch"
SRC_URI:append:mx8mm4 = " file://tasd8mm-display-4.patch"
SRC_URI:append:mx8mm2g4 = " file://tasd8mm-2g-display-4.patch"
SRC_URI:append:mx8mm10 = " file://tasd8mm-display-10.patch"
SRC_URI:append:mx8mm2g10 = " file://tasd8mm-2g-display-10.patch"
SRC_URI:append:mx8mm2g15 = " file://tasd8mm-2g-display-15.patch"
SRC_URI:append:mx8mp7 = " file://tasc8mp-display-7.patch"
SRC_URI:append:mx8mp10 = " file://tasc8mp-display-10.patch"
SRC_URI:append:mx8mp12 = " file://tasc8mp-display-12.patch"
SRC_URI:append:mx8mp7sd = " file://tasd8mp-display-7.patch"
SRC_URI:append:mx8mp4sd = " file://tasd8mp-display-4.patch"
SRC_URI:append:mx8mp10sd = " file://tasd8mp-display-10.patch"
SRC_URI:append:tasv93 = " file://tasv93.patch"
SRC_URI:append:tasv8mp = " file://tasv8mp.patch"

do_deploy:append:mx8m-nxp-bsp () {
    # Deploy u-boot-nodtb.bin and fsl-imx8mq-XX.dtb, to be packaged in boot binary by imx-boot
    if [ -n "${UBOOT_CONFIG}" ]
    then
        for config in ${UBOOT_MACHINE}; do
            i=$(expr $i + 1);
            for type in ${UBOOT_CONFIG}; do
                j=$(expr $j + 1);
                if [ $j -eq $i ]
                then
                    install -d ${DEPLOYDIR}/${BOOT_TOOLS}
                    install -m 0777 ${B}/${config}/arch/arm/dts/${UBOOT_DTB_NAME}  ${DEPLOYDIR}/${BOOT_TOOLS}
                    install -m 0777 ${B}/${config}/u-boot-nodtb.bin  ${DEPLOYDIR}/${BOOT_TOOLS}/u-boot-nodtb.bin-${MACHINE}-${type}
                fi
            done
            unset  j
        done
        unset  i
    fi

}

PACKAGE_ARCH = "${MACHINE_ARCH}"
COMPATIBLE_MACHINE = "(mx6|mx8|mx9)"

UBOOT_NAME:mx6-nxp-bsp = "u-boot-${MACHINE}.bin-${UBOOT_CONFIG}"
UBOOT_NAME:mx8-nxp-bsp = "u-boot-${MACHINE}.bin-${UBOOT_CONFIG}"
